@@ -6,10 +6,11 @@ import doctest
 import logging
 import os
 import sys
+import unittest
 
 from contextlib import contextmanager
 
-from pint.compat import ndarray, unittest, np
+from pint.compat import ndarray, np
 
 from pint import logger, UnitRegistry
 from pint.quantity import _Quantity
@@ -104,7 +105,7 @@ class QuantityTestCase(BaseTestCase):
         if isinstance(m1, ndarray) or isinstance(m2, ndarray):
             np.testing.assert_array_equal(m1, m2, err_msg=msg)
         else:
-            unittest.TestCase.assertEqual(self, m1, m2, msg)
+            self.assertEqual(m1, m2, msg)
 
     def assertQuantityAlmostEqual(self, first, second, rtol=1e-07, atol=0, msg=None):
         if msg is None:
@@ -115,7 +116,7 @@ class QuantityTestCase(BaseTestCase):
         if isinstance(m1, ndarray) or isinstance(m2, ndarray):
             np.testing.assert_allclose(m1, m2, rtol=rtol, atol=atol, err_msg=msg)
         else:
-            self.assertLessEqual(abs(m1 - m2), atol + rtol * abs(m2))
+            self.assertLessEqual(abs(m1 - m2), atol + rtol * abs(m2), msg=msg)
 
 
 def testsuite():
