@@ -18,13 +18,19 @@ import re
 import operator
 from numbers import Number
 from fractions import Fraction
-from collections import Mapping
+
+try:
+    from collections.abc import Mapping
+except ImportError:
+    from collections import Mapping
+
+from logging import NullHandler
 
 import logging
 from token import STRING, NAME, OP, NUMBER
 from tokenize import untokenize
 
-from .compat import string_types, tokenizer, lru_cache, NullHandler, maketrans, NUMERIC_TYPES
+from .compat import string_types, tokenizer, lru_cache, maketrans, NUMERIC_TYPES
 from .formatting import format_unit,siunitx_format_unit
 from .pint_eval import build_eval_tree
 from .errors import DefinitionSyntaxError
@@ -149,7 +155,7 @@ def pi_theorem(quantities, registry=None):
 
         if not registry and any(not key.startswith('[') for key in dims):
             logger.warning('A non dimension was found and a registry was not provided. '
-                           'Assuming that it is a dimension name: {0}.'.format(dims))
+                           'Assuming that it is a dimension name: {}.'.format(dims))
 
         quant.append((name, dims))
         dimensions = dimensions.union(dims.keys())
@@ -253,9 +259,9 @@ class UnitsContainer(Mapping):
         self._d = d
         for key, value in d.items():
             if not isinstance(key, string_types):
-                raise TypeError('key must be a str, not {0}'.format(type(key)))
+                raise TypeError('key must be a str, not {}'.format(type(key)))
             if not isinstance(value, Number):
-                raise TypeError('value must be a number, not {0}'.format(type(value)))
+                raise TypeError('value must be a number, not {}'.format(type(value)))
             if not isinstance(value, float):
                 d[key] = float(value)
         self._hash = hash(frozenset(self._d.items()))
@@ -297,6 +303,9 @@ class UnitsContainer(Mapping):
     def __getitem__(self, key):
         return self._d[key]
 
+    def __contains__(self, key):
+        return key in self._d
+
     def __hash__(self):
         return self._hash
 
@@ -320,20 +329,23 @@ class UnitsContainer(Mapping):
         return self.__format__('')
 
     def __repr__(self):
-        tmp = '{%s}' % ', '.join(["'{0}': {1}".format(key, value)
+        tmp = '{%s}' % ', '.join(["'{}': {}".format(key, value)
                                   for key, value in sorted(self._d.items())])
-        return '<UnitsContainer({0})>'.format(tmp)
+        return '<UnitsContainer({})>'.format(tmp)
 
     def __format__(self, spec):
         return format_unit(self, spec)
 
+    def format_babel(self, spec, **kwspec):
+        return format_unit(self, spec, **kwspec)
+
     def __copy__(self):
         return UnitsContainer(self._d)
 
     def __mul__(self, other):
         d = udict(self._d)
         if not isinstance(other, self.__class__):
-            err = 'Cannot multiply UnitsContainer by {0}'
+            err = 'Cannot multiply UnitsContainer by {}'
             raise TypeError(err.format(type(other)))
         for key, value in other.items():
             d[key] += value
@@ -347,7 +359,7 @@ class UnitsContainer(Mapping):
 
     def __pow__(self, other):
         if not isinstance(other, NUMERIC_TYPES):
-            err = 'Cannot power UnitsContainer by {0}'
+            err = 'Cannot power UnitsContainer by {}'
             raise TypeError(err.format(type(other)))
         d = udict(self._d)
         for key, value in d.items():
@@ -356,7 +368,7 @@ class UnitsContainer(Mapping):
 
     def __truediv__(self, other):
         if not isinstance(other, self.__class__):
-            err = 'Cannot divide UnitsContainer by {0}'
+            err = 'Cannot divide UnitsContainer by {}'
             raise TypeError(err.format(type(other)))
 
         d = udict(self._d)
@@ -372,7 +384,7 @@ class UnitsContainer(Mapping):
 
     def __rtruediv__(self, other):
         if not isinstance(other, self.__class__) and other != 1:
-            err = 'Cannot divide {0} by UnitsContainer'
+            err = 'Cannot divide {} by UnitsContainer'
             raise TypeError(err.format(type(other)))
 
         return self**-1
@@ -409,22 +421,23 @@ class ParserHelper(UnitsContainer):
     @classmethod
     def from_string(cls, input_string):
         return cls._from_string(input_string)
-    
+
     @classmethod
     def eval_token(cls, token, use_decimal=False):
-        token_type = token[0]
-        token_text = token[1]
+        token_type = token.type
+        token_text = token.string
         if token_type == NUMBER:
-            if '.' in token_text or 'e' in token_text:
+            try:
+                return int(token_text)
+            except ValueError:
                 if use_decimal:
                     return Decimal(token_text)
                 return float(token_text)
-            return int(token_text)
         elif token_type == NAME:
             return ParserHelper.from_word(token_text)
         else:
             raise Exception('unknown token type')
-    
+
     @classmethod
     @lru_cache()
     def _from_string(cls, input_string):
@@ -490,14 +503,14 @@ class ParserHelper(UnitsContainer):
         return self.__class__(self.scale, d)
 
     def __str__(self):
-        tmp = '{%s}' % ', '.join(["'{0}': {1}".format(key, value)
+        tmp = '{%s}' % ', '.join(["'{}': {}".format(key, value)
                                   for key, value in sorted(self._d.items())])
-        return '{0} {1}'.format(self.scale, tmp)
+        return '{} {}'.format(self.scale, tmp)
 
     def __repr__(self):
-        tmp = '{%s}' % ', '.join(["'{0}': {1}".format(key, value)
+        tmp = '{%s}' % ', '.join(["'{}': {}".format(key, value)
                                   for key, value in sorted(self._d.items())])
-        return '<ParserHelper({0}, {1})>'.format(self.scale, tmp)
+        return '<ParserHelper({}, {})>'.format(self.scale, tmp)
 
     def __mul__(self, other):
         if isinstance(other, string_types):
@@ -550,17 +563,18 @@ class ParserHelper(UnitsContainer):
 
 
 #: List of regex substitution pairs.
-_subs_re = [(r"([\w\.\-\+\*\\\^])\s+", r"\1 "), # merge multiple spaces
-            (r"({0}) squared", r"\1**2"),  # Handle square and cube
-            (r"({0}) cubed", r"\1**3"),
-            (r"cubic ({0})", r"\1**3"),
-            (r"square ({0})", r"\1**2"),
-            (r"sq ({0})", r"\1**2"),
+_subs_re = [('\N{DEGREE SIGN}', " degree"),
+            (r"([\w\.\-\+\*\\\^])\s+", r"\1 "), # merge multiple spaces
+            (r"({}) squared", r"\1**2"),  # Handle square and cube
+            (r"({}) cubed", r"\1**3"),
+            (r"cubic ({})", r"\1**3"),
+            (r"square ({})", r"\1**2"),
+            (r"sq ({})", r"\1**2"),
             (r"\b([0-9]+\.?[0-9]*)(?=[e|E][a-zA-Z]|[a-df-zA-DF-Z])", r"\1*"),  # Handle numberLetter for multiplication
             (r"([\w\.\-])\s+(?=\w)", r"\1*"),  # Handle space for multiplication
             ]
 
-#: Compiles the regex and replace {0} by a regex that matches an identifier.
+#: Compiles the regex and replace {} by a regex that matches an identifier.
 _subs_re = [(re.compile(a.format(r"[_a-zA-Z][_a-zA-Z0-9]*")), b) for a, b in _subs_re]
 _pretty_table = maketrans('⁰¹²³⁴⁵⁶⁷⁸⁹·⁻', '0123456789*-')
 _pretty_exp_re = re.compile(r"⁻?[⁰¹²³⁴⁵⁶⁷⁸⁹]+(?:\.[⁰¹²³⁴⁵⁶⁷⁸⁹]*)?")
@@ -610,12 +624,35 @@ class SharedRegistryObject(object):
             return True
 
         elif isinstance(other, SharedRegistryObject):
-            mess = 'Cannot operate with {0} and {1} of different registries.'
+            mess = 'Cannot operate with {} and {} of different registries.'
             raise ValueError(mess.format(self.__class__.__name__,
                                          other.__class__.__name__))
         else:
             return False
 
+
+class PrettyIPython(object):
+    """Mixin to add pretty-printers for IPython"""
+
+    def _repr_html_(self):
+        if "~" in self.default_format:
+            return "{:~H}".format(self)
+        else:
+            return "{:H}".format(self)
+
+    def _repr_latex_(self):
+        if "~" in self.default_format:
+            return "${:~L}$".format(self)
+        else:
+            return "${:L}$".format(self)
+
+    def _repr_pretty_(self, p, cycle):
+        if "~" in self.default_format:
+            p.text("{:~P}".format(self))
+        else:
+            p.text("{:P}".format(self))
+
+
 def to_units_container(unit_like, registry=None):
     """ Convert a unit compatible type to a UnitsContainer.
 
@@ -636,14 +673,14 @@ def to_units_container(unit_like, registry=None):
 
 def infer_base_unit(q):
     """Return UnitsContainer of q with all prefixes stripped."""
-    d = {}
+    d = udict()
     parse = q._REGISTRY.parse_unit_name
     for unit_name, power in q._units.items():
         completely_parsed_unit = list(parse(unit_name))[-1]
 
         _, base_unit, __ = completely_parsed_unit
-        d[base_unit] = power
-    return UnitsContainer(d)
+        d[base_unit] += power
+    return UnitsContainer(dict((k, v) for k, v in d.items() if v != 0))  # remove values that resulted in a power of 0
 
 
 def fix_str_conversions(cls):
@@ -691,7 +728,7 @@ class SourceIterator(object):
         line = ''
         while not line or line.startswith('#'):
             lineno, line = next(self.internal)
-            line = line.strip()
+            line = line.split('#', 1)[0].strip()
 
         self.last = lineno, line
         return lineno, line
